@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/hunchentoot/request.lisp,v 1.21 2006/11/05 22:55:06 edi Exp $
+;;; $Header: /usr/local/cvsrep/hunchentoot/request.lisp,v 1.31 2007/03/30 17:13:20 edi Exp $
 
-;;; Copyright (c) 2004-2006, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2007, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -56,7 +56,7 @@ by the client.")
                     :documentation "An alist of the POST parameters
 sent by the client.")
    (script-name :initform nil
-              :documentation "The URI requested by the client without
+                :documentation "The URI requested by the client without
 the query string.")
    (query-string :initform nil
                  :documentation "The query string of this request.")
@@ -90,38 +90,43 @@ supposed to be of content type 'multipart/form-data'."
                            "NAME"
                            (rfc2388:header-parameters content-disposition-header)))
           when name
-            collect (cons name
-                          (let ((contents (rfc2388:mime-part-contents part)))
-                            (if (pathnamep contents)
-                              (list contents
-                                    (rfc2388:get-file-name headers)
-                                    (rfc2388:content-type part :as-string t))
-                              contents))))))
+          collect (cons name
+                        (let ((contents (rfc2388:mime-part-contents part)))
+                          (if (pathnamep contents)
+                            (list contents
+                                  (rfc2388:get-file-name headers)
+                                  (rfc2388:content-type part :as-string t))
+                            contents))))))
 
-(defun get-post-data (&optional (request *request*))
+(defun get-post-data (&key (request *request*) want-stream (already-read 0))
   "Reads the request body from the stream and stores the raw contents
-in the corresponding slot of the REQUEST object."
+\(as an array of octets) in the corresponding slot of the REQUEST
+object.  Returns just the stream if WANT-STREAM is true.  If there's a
+Content-Length header, it is assumed, that ALREADY-READ octets have
+already been read."
   (let* ((headers-in (headers-in request))
-         (content-length (cdr (assoc :content-length headers-in))))
+         (content-length (when-let (content-length-header (cdr (assoc :content-length headers-in)))
+                           (parse-integer content-length-header :junk-allowed t)))
+         (content-stream (content-stream request)))
     (setf (slot-value request 'raw-post-data)
-          (cond (content-length
+          (cond (want-stream content-stream)
+                ((and content-length (> content-length already-read))
+                 (decf content-length already-read)
                  (when (input-chunking-p)
                    ;; see RFC 2616, section 4.4
                    (log-message :warn "Got Content-Length header although input chunking is on."))
-                 (let ((content (make-string (parse-integer content-length
-                                                            :junk-allowed t))))
-                   (read-sequence content (content-stream request))
+                 (let ((content (make-array content-length :element-type 'octet)))
+                   (read-sequence content content-stream)
                    content))
                 ((input-chunking-p)
-                 (let ((buffer (make-string +buffer-length+))
-                       (content (make-array 0 :element-type 'base-char :adjustable t)))
-                   (loop with stream = (content-stream request)
-                         for index = 0 then (+ index pos)
-                         for pos = (read-sequence buffer stream)
-                         do (adjust-array content (+ index pos))
-                            (replace content buffer :start1 index :end2 pos)
-                         while (= pos +buffer-length+))
-                   content))))))
+                 (loop with buffer = (make-array +buffer-length+ :element-type 'octet)
+                       with content = (make-array 0 :element-type 'octet :adjustable t)
+                       for index = 0 then (+ index pos)
+                       for pos = (read-sequence buffer content-stream)
+                       do (adjust-array content (+ index pos))
+                          (replace content buffer :start1 index :end2 pos)
+                       while (= pos +buffer-length+)
+                       finally (return content)))))))
 
 (defmethod initialize-instance :after ((request request) &rest init-args)
   "The only initarg for a REQUEST object is :HEADERS-IN.  All other
@@ -168,37 +173,42 @@ slot values are computed in this :AFTER method."
           ;; if the content-type is 'application/x-www-form-urlencoded'
           ;; or 'multipart/form-data', compute the post parameters from
           ;; the content body
-          (when-let (content-type (cdr (assoc :content-type headers-in)))
-            (multiple-value-bind (type subtype charset)
-                (parse-content-type content-type)
-              (setq post-parameters
-                    (cond ((and (string-equal type "application")
-                                (string-equal subtype "x-www-form-urlencoded"))
-                           (let ((external-format
-                                  (or (when charset
-                                        (handler-case
-                                            (make-external-format (make-keyword charset) :eol-style :lf)
-                                          (error (condition)
-                                            (warn "Ignoring external format of name ~S~
-because of error:~%~A"
-                                                  charset condition))))
-                                      *hunchentoot-default-external-format*)))
-                             (and (or (assoc :content-length headers-in)
-                                      (input-chunking-p))
-                                  (form-url-encoded-list-to-alist
-                                   (split "&" (get-post-data request))
-                                   external-format))))
-                          ((and (string-equal type "multipart")
-                                (string-equal subtype "form-data"))
-                           (setf (slot-value request 'raw-post-data) t)
-                           (handler-case
-                               (let ((*request* request))
-                                 (parse-rfc2388-form-data (content-stream request) content-type))
-                             (error (msg)
-                               (log-message :error
-                                            "While parsing multipart/form-data parameters: ~A"
-                                            msg)
-                               nil))))))))
+          (when (eq (request-method request) :post)
+            (when-let (content-type (cdr (assoc :content-type headers-in)))
+              (multiple-value-bind (type subtype external-format)
+                  (parse-content-type content-type t)
+                (setq post-parameters
+                      (cond ((and (string-equal type "application")
+                                  (string-equal subtype "x-www-form-urlencoded"))
+                             (unless (or (assoc :content-length headers-in)
+                                         (input-chunking-p))
+                               (error "Can't read request body because there's no~
+Content-Length header and input chunking is off."))
+                             (form-url-encoded-list-to-alist
+                              (split "&" (raw-post-data :request request
+                                                        ;; ASCII would suffice according to RFC...
+                                                        :external-format +latin-1+))
+                              external-format))
+                            ((and (string-equal type "multipart")
+                                  (string-equal subtype "form-data"))
+                             (setf (slot-value request 'raw-post-data) t)                           
+                             (handler-case
+                                 (let* ((*request* request)
+                                        (content-stream (content-stream request))
+                                        (start (flexi-stream-position content-stream)))
+                                   (prog1
+                                       (parse-rfc2388-form-data content-stream content-type)
+                                     (let* ((end (flexi-stream-position content-stream))
+                                            (stray-data (get-post-data :already-read (- end start))))
+                                       (when (and stray-data (plusp (length stray-data)))
+                                         (warn "~A octets of stray data after form-data sent by client."
+                                               (length stray-data))))
+                                     (setf (slot-value request 'raw-post-data) t)))
+                               (error (msg)
+                                 (log-message :error
+                                              "While parsing multipart/form-data parameters: ~A"
+                                              msg)
+                                 nil)))))))))
       (error (cond)
         (log-message* "Error when creating REQUEST object: ~A" cond)
         ;; we assume it's not our fault...
@@ -209,18 +219,19 @@ because of error:~%~A"
   "Recomputes the GET and POST parameters for the REQUEST object
 REQUEST.  This only makes sense if you're switching external formats
 during the request."
-  (with-slots (headers-in get-parameters post-parameters raw-post-data query-string)
+  (with-slots (headers-in get-parameters post-parameters query-string)
       request
     (setq get-parameters
           (form-url-encoded-list-to-alist (split "&" query-string) external-format)
           post-parameters
-          (and raw-post-data
-               (when-let (content-type (cdr (assoc :content-type headers-in)))
-                 (multiple-value-bind (type subtype)
-                     (parse-content-type content-type)
-                   (and (string-equal type "application")
-                        (string-equal subtype "x-www-form-urlencoded"))))
-               (form-url-encoded-list-to-alist (split "&" raw-post-data) external-format))))
+          (when-let (raw-post-data (raw-post-data :request request
+                                                  :external-format +latin-1+))
+            (and (when-let (content-type (cdr (assoc :content-type headers-in)))
+                   (multiple-value-bind (type subtype)
+                       (parse-content-type content-type)
+                     (and (string-equal type "application")
+                          (string-equal subtype "x-www-form-urlencoded"))))
+                 (form-url-encoded-list-to-alist (split "&" raw-post-data) external-format)))))
   (values))
                                                 
 (defun script-name (&optional (request *request*))
@@ -381,10 +392,48 @@ TIME."
       (throw 'handler-done nil))
     (values)))
 
-(defun raw-post-data (&optional (request *request*))
-  "Returns the raw string sent as the body of a POST request."
-  (or (slot-value request 'raw-post-data)
-      (get-post-data request)))
+(defun raw-post-data (&key (request *request*) external-format force-binary force-text want-stream)
+  "Returns the content sent by the client if there was any \(unless
+the content type was \"multipart/form-data\").  By default, the result
+is a string if the type of the `Content-Type' media type is \"text\",
+and a vector of octets otherwise.  In the case of a string, the
+external format to be used to decode the content will be determined
+from the `charset' parameter sent by the client \(or otherwise
+*HUNCHENTOOT-DEFAULT-EXTERNAL-FORMAT* will be used).
+
+You can also provide an external format explicitly \(through
+EXTERNAL-FORMAT) in which case the result will unconditionally be a
+string.  Likewise, you can provide a true value for FORCE-TEXT which
+will force Hunchentoot to act as if the type of the media type had
+been \"text\".  Or you can provide a true value for FORCE-BINARY which
+means that you want a vector of octets at any rate.
+
+If, however, you provide a true value for WANT-STREAM, the other
+parameters are ignored and you'll get the content \(flexi) stream to
+read from it yourself.  It is then your responsibility to read the
+correct amount of data, because otherwise you won't be able to return
+a response to the client.  If the content type of the request was
+`multipart/form-data' or `application/x-www-form-urlencoded', the
+content has been read by Hunchentoot already and you can't read from
+the stream anymore.
+
+You can call RAW-POST-DATA more than once per request, but you can't
+mix calls which have different values for WANT-STREAM.
+
+Note that this function is slightly misnamed because a client can send
+content even if the request method is not POST."
+  (when (and force-binary force-text)
+    (error "It doesn't make sense to set both FORCE-BINARY and FORCE-TEXT to a true value."))
+  (unless (or external-format force-binary)
+    (setq external-format
+          (when-let (content-type (cdr (assoc :content-type (headers-in request))))
+            (nth-value 2 (parse-content-type content-type force-text)))))
+  (let ((raw-post-data (or (slot-value request 'raw-post-data)
+                           (get-post-data :request request :want-stream want-stream))))
+    (cond ((typep raw-post-data 'stream) raw-post-data)
+          ((member raw-post-data '(t nil)) nil)
+          (external-format (octets-to-string raw-post-data :external-format external-format))
+          (t raw-post-data))))
 
 (defun aux-request-value (symbol &optional (request *request*))
   "Returns the value associated with SYMBOL from the request object
