@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-USER; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/hunchentoot/server.lisp,v 1.26 2006/11/05 20:27:43 edi Exp $
+;;; $Header: /usr/local/cvsrep/hunchentoot/server.lisp,v 1.35 2007/05/25 11:32:51 edi Exp $
 
-;;; Copyright (c) 2004-2006, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2007, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -40,9 +40,20 @@ See START-SERVER.")
             :reader server-address
             :documentation "The address the server is listening
 on.  See START-SERVER.")
-   (chunkingp :initarg :chunkingp
-              :reader server-chunkingp
-              :documentation "Whether the server may use output chunking.")
+   (name :initarg :name
+         :accessor server-name
+         :documentation "The optional name of the server, a symbol.")
+   (dispatch-table :initarg :dispatch-table
+                   :accessor server-dispatch-table
+                   :documentation "The dispatch-table used by this
+server.  Can be NIL to denote that *META-DISPATCHER* should be called
+instead.")
+   (output-chunking-p :initarg :output-chunking-p
+                      :reader server-output-chunking-p
+                      :documentation "Whether the server may use output chunking.")
+   (input-chunking-p :initarg :input-chunking-p
+                     :reader server-input-chunking-p
+                     :documentation "Whether the server may use input chunking.")
    (read-timeout :initarg :read-timeout
                  :reader server-read-timeout
                  :documentation "The read-timeout of the server.")
@@ -65,14 +76,17 @@ Hunchentoot server or \"just\" infrastructure for mod_lisp.")
                      :reader server-use-apache-log-p
                      :documentation "Whether the server should use
 Apache's log file.  Only applicable if MOD-LISP-P is true.")
+   #-:hunchentoot-no-ssl
    (ssl-certificate-file :initarg :ssl-certificate-file
                          :reader server-ssl-certificate-file
                          :documentation "The namestring of a
 certificate file if SSL is used, NIL otherwise.")
+   #-:hunchentoot-no-ssl
    (ssl-privatekey-file :initarg :ssl-privatekey-file
                         :reader server-ssl-privatekey-file
                         :documentation "The namestring of a
 private key file if SSL is used, NIL otherwise.")
+   #-:hunchentoot-no-ssl
    (ssl-privatekey-password :initarg :ssl-privatekey-password
                             :reader server-ssl-privatekey-password
                             :documentation "The password for the
@@ -87,15 +101,18 @@ information about a running Hunchentoot server instance."))
 
 (defun start-server (&key (port 80 port-provided-p)
                           address
+                          dispatch-table
+                          (name (gensym))
                           (mod-lisp-p nil)
                           (use-apache-log-p mod-lisp-p)
+                          (input-chunking-p t)
                           (read-timeout *default-read-timeout*)
                           (write-timeout *default-write-timeout*)
-                          #+:unix setuid
-                          #+:unix setgid
-                          ssl-certificate-file
-                          (ssl-privatekey-file ssl-certificate-file)
-                          ssl-privatekey-password)
+                          #+(and :unix (not :win32)) setuid
+                          #+(and :unix (not :win32)) setgid
+                          #-:hunchentoot-no-ssl ssl-certificate-file
+                          #-:hunchentoot-no-ssl (ssl-privatekey-file ssl-certificate-file)
+                          #-:hunchentoot-no-ssl ssl-privatekey-password)
   "Starts a Hunchentoot server and returns the SERVER object \(which
 can be stopped with STOP-SERVER).  PORT is the port the server will be
 listening on - the default is 80 \(or 443 if SSL information is
@@ -106,17 +123,34 @@ host names such as \"www.nowhere.com\" and address strings like
 \"204.71.177.75\".  If ADDRESS is NIL, then the server will receive
 connections to all IP addresses on the machine.  This is the default.
 
+DISPATCH-TABLE can either be a dispatch table which is to be used by
+this server or NIL which means that at request time *META-DISPATCHER*
+will be called to retrieve a dispatch table.
+
+NAME should be a symbol which can be used to name the server.  This
+name can utilized when defining \"easy handlers\" - see
+DEFINE-EASY-HANDLER.  The default name is an uninterned symbol as
+returned by GENSYM.
+
 If MOD-LISP-P is true, the server will act as a back-end for mod_lisp,
 otherwise it will be a stand-alone web server.  If USE-APACHE-LOG-P is
 true, log messages will be written to the Apache log file - this
 parameter has no effect if MOD-LISP-P is NIL.
 
-On LispWorks 5.0 or higher READ-TIMEOUT and WRITE-TIMEOUT are the
-read and write timeouts \(in seconds) of the server - use NIL for
-no timeout at all.  (See the LispWorks documentation for
-STREAM:SOCKET-STREAM for details.)  On LispWorks 4.4.6 or lower,
-SBCL, and CMUCL WRITE-TIMEOUT is ignored.  On AllegroCL and
-OpenMCL both parameters are ignored.
+If INPUT-CHUNKING-P is true, the server will accept request bodies
+without a `Content-Length' header if the client uses chunked transfer
+encoding.  If you want to use this feature together with mod_lisp, you
+should make sure that your combination of Apache and mod_lisp can do
+that - see:
+
+  <http://common-lisp.net/pipermail/mod-lisp-devel/2006-December/000104.html>.
+
+On LispWorks 5.0 or higher and AllegroCL, READ-TIMEOUT and
+WRITE-TIMEOUT are the read and write timeouts \(in seconds) of
+the server - use NIL for no timeout at all.  (See the LispWorks
+documentation for STREAM:SOCKET-STREAM for details.)  On
+LispWorks 4.4.6 or lower, SBCL, and CMUCL WRITE-TIMEOUT is
+ignored.  On OpenMCL both parameters are ignored.
 
 On Unix you can use SETUID and SETGID to change the UID and GID of the
 process directly after the server has been started.  \(You might want
@@ -136,10 +170,11 @@ associated with a password."
   ;; initialize the session secret if needed
   (unless (boundp '*session-secret*)
     (reset-session-secret))
-  (let ((chunkingp t))
+  (let ((output-chunking-p t))
+    #-:hunchentoot-no-ssl
     (when ssl-certificate-file
-      ;; disable chunking for SSL connections
-      (setq chunkingp nil)
+      ;; disable output chunking for SSL connections
+      (setq output-chunking-p nil)
       (unless port-provided-p (setq port 443)))
     ;; no timeouts if behind mod_lisp
     (when mod-lisp-p
@@ -151,11 +186,17 @@ associated with a password."
     (let ((server (make-instance 'server
                                  :port port
                                  :address address
-                                 :chunkingp (and chunkingp (not mod-lisp-p))
+                                 :name name
+                                 :dispatch-table dispatch-table
+                                 :output-chunking-p (and output-chunking-p (not mod-lisp-p))
+                                 :input-chunking-p input-chunking-p
+                                 #-:hunchentoot-no-ssl #-:hunchentoot-no-ssl
                                  :ssl-certificate-file (and ssl-certificate-file
                                                             (namestring ssl-certificate-file))
+                                 #-:hunchentoot-no-ssl #-:hunchentoot-no-ssl
                                  :ssl-privatekey-file (and ssl-privatekey-file
                                                            (namestring ssl-privatekey-file))
+                                 #-:hunchentoot-no-ssl #-:hunchentoot-no-ssl
                                  :ssl-privatekey-password ssl-privatekey-password
                                  :mod-lisp-p mod-lisp-p
                                  :use-apache-log-p (and mod-lisp-p use-apache-log-p)
@@ -202,14 +243,14 @@ associated with a password."
                (setf (server-listener server) process))
               (condition
                (error condition))))
-      #+:unix
+      #+(and :unix (not :win32))
       (when setgid
         ;; we must make sure to call setgid before we call setuid or
         ;; suddenly we aren't root anymore...
         (etypecase setgid
           (integer (setgid setgid))
           (string (setgid (get-gid-from-name setgid)))))
-      #+:unix
+      #+(and :unix (not :win32))
       (when setuid
         (etypecase setuid
           (integer (setuid setuid))
@@ -257,6 +298,7 @@ closed or until a read timeout occurs."
                                       (server-read-timeout server)
                                       (server-write-timeout server)))
               ;; attach SSL to the stream if necessary
+              #-:hunchentoot-no-ssl
               (when (server-ssl-certificate-file server)
                 #+:lispworks
                 (make-ssl-server-stream *hunchentoot-stream*
@@ -268,18 +310,20 @@ closed or until a read timeout occurs."
                         (cl+ssl:make-ssl-server-stream *hunchentoot-stream*
                                                        :certificate (server-ssl-certificate-file server)
                                                        :key (server-ssl-privatekey-file server))))
-              ;; wrap with chunking-enabled stream if we're not behind mod_lisp
-              (unless (server-mod-lisp-p server)
+              ;; wrap with chunking-enabled stream if necessary
+              (when (or (server-input-chunking-p server)
+                        (server-output-chunking-p server))
                 (setq *hunchentoot-stream* (make-chunked-stream *hunchentoot-stream*)))
               ;; now wrap with flexi stream with "faithful" external format
               (setq *hunchentoot-stream*
-                    (make-flexi-stream *hunchentoot-stream*
-                                       :external-format '(:latin1 :eol-style :lf)))
+                    (make-flexi-stream *hunchentoot-stream* :external-format +latin-1+))
               ;; loop until we have to close the stream - as
               ;; determined by *CLOSE-HUNCHENTOOT-STREAM*
               (unwind-protect                  
                   (loop
                    (let ((*close-hunchentoot-stream* t))
+                     ;; reset to "faithful" format on each iteration
+                     (setf (flexi-stream-external-format *hunchentoot-stream*) +latin-1+)
                      (multiple-value-bind (headers-in content-stream method url-string server-protocol)
                          (get-request-data)
                        (unless (and ;; check if there was a request at all
@@ -288,8 +332,9 @@ closed or until a read timeout occurs."
                                     (prog1
                                         (process-request headers-in content-stream method
                                                          url-string server-protocol)
-                                      (unless (server-mod-lisp-p server)
-                                        ;; always turn chunking off at this point
+                                      ;; always turn chunking off at this point
+                                      (when (or (server-input-chunking-p server)
+                                                (server-output-chunking-p server))
                                         (setf (chunked-stream-output-chunking-p
                                                (flexi-stream-stream *hunchentoot-stream*)) nil
                                               (chunked-stream-input-chunking-p
@@ -315,19 +360,8 @@ dispatches to a handler, and finally sends the output to the client
 using START-OUTPUT.  If all goes as planned, the function returns T."
   (let (*tmp-files* *headers-sent*)
     (unwind-protect
-        (let* ((*session* nil)
-               ;; first create a REPLY object so we can immediately start
-               ;; logging \(in case we're logging to mod_lisp)
-               (*reply* (make-instance 'reply))
-               (*request* (make-instance 'request
-                                         :headers-in headers-in
-                                         :content-stream content-stream
-                                         :method method
-                                         :uri url-string
-                                         :server-protocol server-protocol))
-               (*dispatch-table* (funcall *meta-dispatcher* *server*))
-               backtrace)
-          (unless (server-mod-lisp-p *server*)
+        (progn
+          (when (server-input-chunking-p *server*)
             (let ((transfer-encodings (cdr (assoc :transfer-encoding headers-in))))
               (when transfer-encodings
                 (setq transfer-encodings
@@ -336,57 +370,68 @@ using START-OUTPUT.  If all goes as planned, the function returns T."
                 ;; turn chunking on before we read the request body
                 (setf (chunked-stream-input-chunking-p 
                        (flexi-stream-stream *hunchentoot-stream*)) t))))
-          (multiple-value-bind (body error)
-              (catch 'handler-done
-                (handler-bind ((error
-                                (lambda (cond)
-                                  ;; only generate backtrace if needed
-                                  (setq backtrace
-                                        (and (or (and *show-lisp-errors-p*
-                                                      *show-lisp-backtraces-p*)
-                                                 (and *log-lisp-errors-p*
-                                                      *log-lisp-backtraces-p*))
-                                             (get-backtrace cond)))
-                                  (when *log-lisp-errors-p*
-                                    (log-message *lisp-errors-log-level*
-                                                 "~A~:[~*~;~%~A~]"
-                                                 cond
-                                                 *log-lisp-backtraces-p*
-                                                 backtrace))
-                                  ;; if the headers were already sent
-                                  ;; the error happens within the body
-                                  ;; and we have to close the stream
-                                  (when *headers-sent*
-                                    (setq *close-hunchentoot-stream* t))
-                                  (throw 'handler-done
-                                         (values nil cond))))
-                               (warning
-                                (lambda (cond)
-                                  (when *log-lisp-warnings-p*
-                                    (log-message *lisp-warnings-log-level*
-                                                 "~A~:[~*~;~%~A~]"
-                                                 cond
-                                                 *log-lisp-backtraces-p*
-                                                 backtrace)))))
-                  (with-debugger 
-                    ;; skip dispatch if bad request
-                    (when (eq (return-code) +http-ok+)
-                      ;; read post data to clear stream
-                      (raw-post-data)
-                      ;; now do the work
-                      (dispatch-request *dispatch-table*)))))
-            (when error
-              (setf (return-code *reply*)
+          (let* ((*session* nil)
+                 ;; first create a REPLY object so we can immediately start
+                 ;; logging \(in case we're logging to mod_lisp)
+                 (*reply* (make-instance 'reply))
+                 (*request* (make-instance 'request
+                                           :headers-in headers-in
+                                           :content-stream content-stream
+                                           :method method
+                                           :uri url-string
+                                           :server-protocol server-protocol))
+                 (*dispatch-table* (or (server-dispatch-table *server*)
+                                       (funcall *meta-dispatcher* *server*)))
+                 backtrace)
+            (multiple-value-bind (body error)
+                (catch 'handler-done
+                  (handler-bind ((error
+                                  (lambda (cond)
+                                    ;; only generate backtrace if needed
+                                    (setq backtrace
+                                          (and (or (and *show-lisp-errors-p*
+                                                        *show-lisp-backtraces-p*)
+                                                   (and *log-lisp-errors-p*
+                                                        *log-lisp-backtraces-p*))
+                                               (get-backtrace cond)))
+                                    (when *log-lisp-errors-p*
+                                      (log-message *lisp-errors-log-level*
+                                                   "~A~:[~*~;~%~A~]"
+                                                   cond
+                                                   *log-lisp-backtraces-p*
+                                                   backtrace))
+                                    ;; if the headers were already sent
+                                    ;; the error happens within the body
+                                    ;; and we have to close the stream
+                                    (when *headers-sent*
+                                      (setq *close-hunchentoot-stream* t))
+                                    (throw 'handler-done
+                                           (values nil cond))))
+                                 (warning
+                                  (lambda (cond)
+                                    (when *log-lisp-warnings-p*
+                                      (log-message *lisp-warnings-log-level*
+                                                   "~A~:[~*~;~%~A~]"
+                                                   cond
+                                                   *log-lisp-backtraces-p*
+                                                   backtrace)))))
+                    (with-debugger 
+                      ;; skip dispatch if bad request
+                      (when (eq (return-code) +http-ok+)
+                        ;; now do the work
+                        (dispatch-request *dispatch-table*)))))
+              (when error
+                (setf (return-code *reply*)
                       +http-internal-server-error+))
-            (start-output (cond ((and error *show-lisp-errors-p*)
-                                 (format nil "<pre>~A~:[~*~;~%~%~A~]</pre>"
-                                         (escape-for-html (format nil "~A" error))
-                                         *show-lisp-backtraces-p*
-                                         (escape-for-html (format nil "~A" backtrace))))
-                                (error
-                                 "An error has occured")
-                                (t body))))
-          t)
+              (start-output (cond ((and error *show-lisp-errors-p*)
+                                   (format nil "<pre>~A~:[~*~;~%~%~A~]</pre>"
+                                           (escape-for-html (format nil "~A" error))
+                                           *show-lisp-backtraces-p*
+                                           (escape-for-html (format nil "~A" backtrace))))
+                                  (error
+                                   "An error has occured")
+                                  (t body))))
+            t))
       (dolist (path *tmp-files*)
         (when (and (pathnamep path) (probe-file path))
           (ignore-errors (delete-file path)))))))
