@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/hunchentoot/test/test.lisp,v 1.14 2006/10/09 22:57:06 edi Exp $
+;;; $Header: /usr/local/cvsrep/hunchentoot/test/test.lisp,v 1.18 2007/04/02 21:21:19 edi Exp $
 
-;;; Copyright (c) 2004-2006, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2007, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -47,11 +47,14 @@
 
 (defmacro with-lisp-output ((var) &body body)
   `(let ((*package* (find-package :hunchentoot-test-user)))
-     (with-output-to-string (,var)
+     (with-output-to-string (,var #+:lispworks nil
+                                  #+:lispworks :element-type
+                                  #+:lispworks 'lw:simple-char)
        ,@body)))
 
 (defmacro info-table (&rest forms)
-  (let ((=value= (gensym)))
+  (let ((=value= (gensym))
+        (=first= (gensym)))
     `(with-html-output (*standard-output*)
        (:p (:table :border 1 :cellpadding 2 :cellspacing 0
             (:tr (:td :colspan 2
@@ -66,8 +69,8 @@
                                (:pre :style "padding: 0px"
                                 (esc (with-lisp-output (s)
                                        (loop for ,=value= in (multiple-value-list ,form)
-                                             for first = t then nil
-                                             unless first
+                                             for ,=first= = t then nil
+                                             unless ,=first=
                                              do (princ ", " s)
                                              do (pprint ,=value= s))))))))))
        (menu-link))))
@@ -474,6 +477,8 @@ and see what's happening.")
   (with-html
     (:html
      (:head
+      (:link :rel "shortcut icon"
+       :href "/hunchentoot/test/favicon.ico" :type "image/x-icon")
       (:title "Hunchentoot test menu"))
      (:body
       (:h2 (str *headline*))
@@ -544,6 +549,10 @@ and see what's happening.")
               (make-pathname :name "fz" :type "jpg" :version nil
                              :defaults *this-file*)
               "image/jpeg")
+             (create-static-file-dispatcher-and-handler
+              "/hunchentoot/test/favicon.ico"
+              (make-pathname :name "favicon" :type "ico" :version nil
+                             :defaults *this-file*))
              (create-folder-dispatcher-and-handler
               "/hunchentoot/code/"
               (make-pathname :name nil :type nil :version nil
