@@ -1,7 +1,7 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/hunchentoot/util.lisp,v 1.18 2006/10/09 22:57:03 edi Exp $
+;;; $Header: /usr/local/cvsrep/hunchentoot/util.lisp,v 1.30 2007/05/25 16:53:01 edi Exp $
 
-;;; Copyright (c) 2004-2006, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2004-2007, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -100,10 +100,15 @@ are discarded \(that is, the body is an implicit PROGN)."
                             `(let (,,@temps)
                               ,,@body))))))
 
-(defun maybe-invoke-debugger (condition)
-  "Invokes the debugger with CONDITION if *CATCH-ERRORS-P* is NIL."
-  (unless *catch-errors-p*
-    (invoke-debugger condition)))
+(defgeneric maybe-invoke-debugger (condition)
+  (:documentation "This generic function is called whenever a
+condition CONDITION is signaled in Hunchentoot.  You might want to
+specialize it on specific condition classes for debugging purposes.")
+  (:method (condition)
+   "The default method invokes the debugger with CONDITION if
+*CATCH-ERRORS-P* is NIL."
+   (unless *catch-errors-p*
+     (invoke-debugger condition))))
 
 (defmacro with-debugger (&body body)
   "Executes BODY and invokes the debugger if an error is signaled and
@@ -150,51 +155,11 @@ digits."
 cease to be valid."
   (setq *session-secret* (create-random-string 10 36)))
 
-(defun status-line (return-code)
-  "Returns a meaningful status line for the http return code
-RETURN-CODE \(which should be an integer) or NIL for uncommon return
-codes."
-  (case return-code
-    ((#.+http-ok+) "OK")
-    ((#.+http-moved-permanently+) "Moved Permanently")
-    ((#.+http-moved-temporarily+) "Moved Temporarily")
-    ((#.+http-not-modified+) "Not Modified")
-    ((#.+http-authorization-required+) "Authorization Required")
-    ((#.+http-forbidden+) "Forbidden")
-    ((#.+http-not-found+) "Not Found")
-    ((#.+http-internal-server-error+) "Internal Server Error")
-    ((#.+http-continue+) "Continue")
-    ((#.+http-switching-protocols+) "Switching Protocols")
-    ((#.+http-created+) "Created")
-    ((#.+http-accepted+) "Accepted")
-    ((#.+http-non-authoritative-information+) "Non-Authoritative Information")
-    ((#.+http-no-content+) "No Content")
-    ((#.+http-reset-content+) "Reset Content")
-    ((#.+http-partial-content+) "Partial Content")
-    ((#.+http-multiple-choices+) "Multiple Choices")
-    ((#.+http-see-other+) "See Other")
-    ((#.+http-use-proxy+) "Use Proxy")
-    ((#.+http-temporary-redirect+) "Temporary Redirect")
-    ((#.+http-bad-request+) "Bad Request")
-    ((#.+http-payment-required+) "Payment Required")
-    ((#.+http-method-not-allowed+) "Method Not Allowed")
-    ((#.+http-not-acceptable+) "Not Acceptable")
-    ((#.+http-proxy-authentication-required+) "Proxy Authentication Required")
-    ((#.+http-request-time-out+) "Request Time-out")
-    ((#.+http-conflict+) "Conflict")
-    ((#.+http-gone+) "Gone")
-    ((#.+http-length-required+) "Length Required")
-    ((#.+http-precondition-failed+) "Precondition Failed")
-    ((#.+http-request-entity-too-large+) "Request Entity Too Large")
-    ((#.+http-request-uri-too-large+) "Request-URI Too Large")
-    ((#.+http-unsupported-media-type+) "Unsupported Media Type")
-    ((#.+http-requested-range-not-satisfiable+) "Requested range not satisfiable")
-    ((#.+http-expectation-failed+) "Expectation Failed")
-    ((#.+http-not-implemented+) "Not Implemented")
-    ((#.+http-bad-gateway+) "Bad Gateway")
-    ((#.+http-service-unavailable+) "Service Unavailable")
-    ((#.+http-gateway-time-out+) "Gateway Time-out")
-    ((#.+http-version-not-supported+) "Version not supported")))
+(defun reason-phrase (return-code)
+  "Returns a reason phrase for the HTTP return code RETURN-CODE
+\(which should be an integer) or NIL for return codes Hunchentoot
+doesn't know."
+  (gethash return-code *http-reason-phrase-map*))
 
 (defun make-keyword (string &key (destructivep t))
   "Interns the upcased version of STRING into the KEYWORD package.
@@ -359,8 +324,6 @@ alist.  Both names and values are url-decoded while doing this."
                         ;; note that there's no comma in there - because of cookies
                         (find c "$-_.!*'()" :test #'char=))
                      (write-char c s))
-                   ((char= c #\Space)
-                     (write-char #\+ s))
                    (t (loop for octet across (string-to-octets string
                                                                :start index
                                                                :end (1+ index)
@@ -375,18 +338,38 @@ alist.  Both names and values are url-decoded while doing this."
                        *force-output-timeout*))
     (force-output stream)))
 
-(defun parse-content-type (content-type-header)
+(defun parse-content-type (content-type-header &optional want-external-format-p)
   "Reads and parses a `Content-Type' header and returns it as three
-values - the type, the subtype, and the 'charset' parameter, if there
-is one.  CONTENT-TYPE-HEADER is supposed to be the corresponding
-header value as a string."
+values - the type, the subtype, and an external format corresponding
+to the 'charset' parameter in the header \(or
+*HUNCHENTOOT-DEFAULT-EXTERNAL-FORMAT*), if there is one and if the
+content type is \"text\" or WANT-EXTERNAL-FORMAT-P is true.
+CONTENT-TYPE-HEADER is supposed to be the corresponding header value
+as a string."
   (with-input-from-string (stream content-type-header)
     (let* ((*current-error-message* "Corrupted Content-Type header:")
            (type (read-token stream))
-           (subtype (and (assert-char stream #\/)
+           (subtype (and (or (ignore-errors (assert-char stream #\/))
+                             (return-from parse-content-type
+                               ;; try to return something meaningful
+                               (values "application" "octet-stream"
+                                       (and want-external-format-p
+                                            *hunchentoot-default-external-format*))))
                          (read-token stream)))
-           (parameters (read-name-value-pairs stream)))
-      (values type subtype (cdr (assoc "charset" parameters))))))
+           (parameters (read-name-value-pairs stream))
+           (charset (cdr (assoc "charset" parameters)))
+           (external-format
+            (and (or want-external-format-p
+                     (string-equal type "text"))
+                 (or (when charset
+                       (handler-case
+                           (make-external-format (make-keyword charset) :eol-style :lf)
+                         (error (condition)
+                           (warn "Ignoring external format of name ~S~
+because of error:~%~A"
+                                 charset condition))))
+                     *hunchentoot-default-external-format*))))
+      (values type subtype external-format))))
 
 (defun get-token-and-parameters (header)
   (with-input-from-string (stream header)
@@ -430,7 +413,7 @@ inclusion in HTML output."
           (scan ":\\d+$" (or (host *request*) ""))
           (server-port)))
 
-(defun server-name ()
+(defun server-name-header ()
   "Returns a string which can be used for 'Server' headers."
   (format nil "Hunchentoot ~A" *hunchentoot-version*))
 
@@ -442,7 +425,7 @@ stream."
   (chunked-stream-input-chunking-p (flexi-stream-stream *hunchentoot-stream*)))
 
 (defun cleanup-function ()
-  "The default for *CLEANUP-FUNCTION*.  Invokes a GC on LispWorks and
-does nothing on other Lisps."
-  #+:lispworks
-  (hcl:mark-and-sweep 2))
\ No newline at end of file
+  "The default for *CLEANUP-FUNCTION*.  Invokes a GC on 32-bit
+LispWorks and does nothing on other Lisps."
+  #+(and :lispworks (not :lispworks-64bit))
+  (hcl:mark-and-sweep 2))
